<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Exception;
use FiloBlu\OrderAdditionalData\Model\ItemDataFactory;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use function serialize;

/**
 * Class ItemSave
 * Save Additional Data when saving Order Items by Repository
 * @package FiloBlu\OrderAdditionalData\Model\Plugin\
 */
class ItemSave
{
    /** @var ItemDataFactory */
    protected $itemDataFactory;

    /**
     * ItemSave constructor.
     * @param ItemDataFactory $itemDataFactory
     */
    public function __construct(ItemDataFactory $itemDataFactory)
    {
        $this->itemDataFactory = $itemDataFactory;
    }

    /**
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemInterface $resultItem
     * @return OrderItemInterface
     * @throws CouldNotSaveException
     */
    public function afterSave(
        OrderItemRepositoryInterface $subject,
        OrderItemInterface $resultItem
    )
    {
        /** @var OrderInterface $resultItem */
        return $this->saveItemAdditionalData($resultItem);
    }

    /**
     * @param OrderItemInterface $item
     * @return OrderItemInterface
     * @throws CouldNotSaveException
     */
    protected function saveItemAdditionalData(OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();

        if (
            null !== $extensionAttributes &&
            null !== $extensionAttributes->getFbOad()
        ) {
            $additionalDataObject = $extensionAttributes->getFbOad();
            $additionalData = [];
            foreach ($additionalDataObject->getCustomAttributes() as $customAttribute) {
                $additionalData[$customAttribute->getAttributeCode()] = $customAttribute->getValue();
            }
            try {
                $itemData = $this->itemDataFactory->create()->load($item->getItemId(), 'item_id');
                $itemData->setItemId($item->getItemId());
                $itemData->setAdditionalData($this->serialize($additionalData));
                $itemData->save();
            } catch (Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for item: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $item;
    }

    /**
     * @param $data
     * @return mixed
     */
    public function serialize($data)
    {
        if (class_exists(SerializerInterface::class)) {
            return ObjectManager::getInstance()->create(SerializerInterface::class)
                ->serialize($data);
        }

        return serialize($data);
    }
}
