<?php

namespace FiloBlu\OrderAdditionalData\Setup;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\DB\AggregatedFieldDataConverter;
use Magento\Framework\DB\DataConverter\SerializedToJson;
use Magento\Framework\DB\FieldToConvert;
use Magento\Framework\DB\Query\Generator;
use Magento\Framework\DB\Select\QueryModifier;
use Magento\Framework\DB\Select\QueryModifierFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;

/**
 * Class UpgradeData
 * @package FiloBlu\OrderAdditionalData\Setup
 */
class UpgradeData implements UpgradeDataInterface
{
    /**
     * {@inheritdoc}
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        if (version_compare($context->getVersion(), '1.1.6', '<')) {
            $this->convertSerializedDataToJson($setup);
        }
    }

    /**
     * Upgrade to version 1.1.6,
     * convert data for the sales_oad_order.additional_data and sales_oad_order.additional_data
     * from serialized to JSON format.
     * Convert also configuration data
     * (oad/general/order_data_fields and oad/general/item_data_fields in core_config_data table).
     *
     * @param ModuleDataSetupInterface $setup
     * @return void
     */
    private function convertSerializedDataToJson(ModuleDataSetupInterface $setup)
    {
        if (!class_exists(QueryModifier::class)) {
            return;
        }

        $fieldsToUpdate[] = new FieldToConvert(
            SerializedToJson::class,
            $setup->getTable('sales_oad_order'),
            'data_id',
            'additional_data'
        );
        $fieldsToUpdate[] = new FieldToConvert(
            SerializedToJson::class,
            $setup->getTable('sales_oad_order_item'),
            'data_id',
            'additional_data'
        );

        $queryModifierFactory = ObjectManager::getInstance()->get(QueryModifierFactory::class);

        // Convert data for the configuration
        $queryModifier = $queryModifierFactory->create(
            'in',
            [
                'values' => [
                    'path' => [
                        'oad/general/order_data_fields', 'oad/general/item_data_fields'
                    ]
                ]
            ]
        );

        $fieldsToUpdate[] = new FieldToConvert(
            SerializedToJson::class,
            $setup->getTable('core_config_data'),
            'config_id',
            'value',
            $queryModifier
        );
    }
}


